/*
 * Copyright (C) 1999 ARM Limited
 * Copyright (C) 2000 Deep Blue Solutions Ltd
 * Copyright 2006-2012 Freescale Semiconductor, Inc.
 * Copyright 2008 Juergen Beisert, kernel@pengutronix.de
 * Copyright 2009 Ilya Yanok, Emcraft Systems Ltd, yanok@emcraft.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/* This file was modified by BAM Labs, Inc.
 *
 * Modified the system reset to output additional information
 * before initializing data and going into loop waiting for the
 * watchdog reset to assert.
 */

#include <linux/kernel.h>
#include <linux/clk.h>
#include <linux/io.h>
#include <linux/err.h>
#include <linux/delay.h>
#include <linux/string.h>
#include <mach/hardware.h>
#include <mach/common.h>
#include <mach/system.h>
#include <asm/proc-fns.h>
#include <asm/system.h>
#ifdef CONFIG_SMP
#include <linux/smp.h>
#endif
#include <asm/mach-types.h>

static void __iomem *wdog_base;


static void arch_reset_special_mode(char mode, const char *cmd)
{
	if (strcmp(cmd, "download") == 0)
		do_switch_mfgmode();
	else if (strcmp(cmd, "recovery") == 0)
		do_switch_recovery();
	else if (strcmp(cmd, "fastboot") == 0)
		do_switch_fastboot();
}

/*
 * Reset the system. It is called by machine_restart().
 */
void arch_reset(char mode, const char *cmd)
{
	unsigned int wcr_enable;
	void __iomem *src_base;
	extern void bamse_reset(void);

	src_base = ioremap(SRC_BASE_ADDR, PAGE_SIZE);
printk("SRC_BASE 0x%x\n", __raw_readl(src_base));
printk("SRC_BASE+4 0x%08x\n", __raw_readl(src_base+4));
printk("SRC_BASE+8 0x%08x\n", __raw_readl(src_base+8));
printk("SRC_BASE+0x14 0x%08x\n", __raw_readl(src_base+0x14));
printk("SRC_BASE+0x18 0x%08x\n", __raw_readl(src_base+0x18));
printk("SRC_BASE+0x1c 0x%08x\n", __raw_readl(src_base+0x1c));


	arch_reset_special_mode(mode, cmd);

#ifdef CONFIG_ARCH_MX6
	/* wait for reset to assert... */
	#ifdef CONFIG_MX6_INTER_LDO_BYPASS
	wcr_enable = 0x14; /*reset system by extern pmic*/
	#else
	wcr_enable = (1 << 2);
	#endif

	for (;;) {
		uint pll_regs;
		void __iomem *pll_base;
#if 0
		int i;
#endif

		__asm__ volatile ("cpsid i");

		pll_base = ioremap(0x20c4000, PAGE_SIZE);

		pll_regs = 0x20c4000;

		/* Need to set CCM_CCSR, but requires certain
		 * conditions to be met.
		__raw_writel(0x100, pll_regs + 0x0c);
		 */
		__raw_writel(0x33f71f92, pll_base + 0x30); /* CCM_CDCDR */
		__raw_writel(0x0002a150, pll_base + 0x34);
		__raw_writel(0x00029150, pll_base + 0x38);
		__raw_writel(0x00080079, pll_base + 0x54);
		__raw_writel(0x000a0001, pll_base + 0x60);
		__raw_writel(0x00c0003f, pll_base + 0x68);
		__raw_writel(0x0030fc00, pll_base + 0x6c);
		__raw_writel(0x0fffc3c0, pll_base + 0x70);
		__raw_writel(0x3ff00000, pll_base + 0x74);
		__raw_writel(0x00fff300, pll_base + 0x78);
		__raw_writel(0x0f0000c3, pll_base + 0x7c);
		__raw_writel(0x000003fc, pll_base + 0x80);

#if 0
		for (i=0; i<35; i++) {
			if ((i%4) == 0)
				printk("\n%08x: ", pll_regs + (i*4));
			printk(" %08x", __raw_readl(pll_regs + (i*4)));
		}
		printk("\n");
#endif

	//  wcr_enable = 0x0c;
	  wcr_enable = 0x24;
		__raw_writew(wcr_enable, wdog_base);
		/* errata TKT039676, SRS bit may be missed when
		SRC sample it, need to write the wdog controller
		twice to avoid it */
		__raw_writew(wcr_enable, wdog_base);

		/* wait for reset to assert... */
		mdelay(500);
		__raw_writew(wcr_enable, wdog_base);
		mdelay(500);

		printk(KERN_ERR "Watchdog reset failed to assert reset\n");
	}

	return;
#endif

#ifdef CONFIG_MACH_MX51_EFIKAMX
	if (machine_is_mx51_efikamx()) {
		mx51_efikamx_reset();
		return;
	}
#endif

	if (cpu_is_mx1()) {
		wcr_enable = (1 << 0);
	} else {
		struct clk *clk;

		clk = clk_get_sys("imx2-wdt.0", NULL);
		if (!IS_ERR(clk))
			clk_enable(clk);
		wcr_enable = (1 << 2);
	}

	/* Assert SRS signal */
	__raw_writew(wcr_enable, wdog_base);

	/* wait for reset to assert... */
	mdelay(500);

	printk(KERN_ERR "Watchdog reset failed to assert reset\n");

	/* delay to allow the serial port to show the message */
	mdelay(50);

	/* we'll take a jump through zero as a poor second */
	cpu_reset(0);
}

void mxc_arch_reset_init(void __iomem *base)
{
	wdog_base = base;
}
